@@ -10,51 +10,6 @@ double c_det(double *matrix, int *rows);
 
 /* -------------------- R level interfaces to LAPACK -------------------- */
 
-/* Singular Value Decomposition (SVD). */
-SEXP r_svd(SEXP matrix) {
-
-SEXP U, D, Vt, result, elnames;
-int nr = 0, nc = 0, mindim = 0, errcode = 0;
-short int duplicated = 0;
-
-  /* compute the dimensions of the SVD components. */
-  nr = nrows(matrix);
-  nc = ncols(matrix);
-  mindim = imin2(nr, nc);
-
-  /* allocate the U, D and t(V) matrices. */
-  PROTECT(U = allocMatrix(REALSXP, nr, mindim));
-  PROTECT(Vt = allocMatrix(REALSXP, mindim, nc));
-  PROTECT(D = allocVector(REALSXP, mindim));
-
-  /* duplicate the matrix so the original one is not overwritten. */
-  if ((duplicated = NAMED(matrix)) > 0)
-    PROTECT(matrix = duplicate(matrix));
-
-  /* call Lapack to do the grunt work. */
-  c_svd(REAL(matrix), REAL(U), REAL(D), REAL(Vt), &nr, &nc, &mindim,
-    TRUE, &errcode);
-
-  /* build the return value. */
-  PROTECT(result = allocVector(VECSXP, 3));
-  PROTECT(elnames = allocVector(STRSXP, 3));
-  SET_STRING_ELT(elnames, 0, mkChar("d"));
-  SET_STRING_ELT(elnames, 1, mkChar("u"));
-  SET_STRING_ELT(elnames, 2, mkChar("vt"));
-  setAttrib(result, R_NamesSymbol, elnames);
-  SET_VECTOR_ELT(result, 0, D);
-  SET_VECTOR_ELT(result, 1, U);
-  SET_VECTOR_ELT(result, 2, Vt);
-
-  if (duplicated > 0)
-    UNPROTECT(6);
-  else
-    UNPROTECT(5);
-
-  return result;
-
-}/*R_SVD*/
-
 /* Determinant. */
 SEXP r_det(SEXP matrix, int scale) {
 
