@@ -291,7 +291,7 @@ SEXP result;
         RandomPermutation(num, perm, work);
 
         for (k = 0; k < num; k++)
-          yperm[k] = yy[perm[k]];
+          yperm[k] = yy[perm[k] - 1];
 
         if (fabs(_cov(xx, yperm, &xm, &ym, &num)) > fabs(observed)) {
 
@@ -395,7 +395,7 @@ SEXP result;
         RandomPermutation(*num, perm, work);
 
         for (k = 0; k < *num; k++)
-          yperm[k] = yorig[perm[k]];
+          yperm[k] = yorig[perm[k] - 1];
 
         /* restore the covariance matrix from the good copy. */
         memcpy(covariance, covariance_backup, ncols * ncols * sizeof(double));
@@ -442,14 +442,8 @@ int i = 0, j = 0;
 double res = 0;
 
   for (i = 0; i < *nrows; i++)
-    for (j = 0; j < *ncols; j++) {
-
-      if (n[CMC(i, j, *nrows)] != 0)
-        res += ((double)n[CMC(i, j, *nrows)]) *
-                 ( log((double)n[CMC(i, j, *nrows)]) + log((double)(*length)) -
-                   log((double)nrowt[i]) - log((double)ncolt[j]) );
-
-    }/*FOR*/
+    for (j = 0; j < *ncols; j++)
+      res += MI_PART(n[CMC(i, j, *nrows)], nrowt[i], ncolt[j], *length); 
 
   return res;
 
@@ -464,17 +458,8 @@ double res = 0;
 
   for (k = 0; k < *nl; k++)
     for (j = 0; j < *nc; j++)
-      for (i = 0; i < *nr; i++) {
-
-       if (n[k][CMC(i, j, *nr)] != 0) {
-
-          res += (double)n[k][CMC(i, j, *nr)] *
-                   ( log((double)n[k][CMC(i, j, *nr)]) + log((double)ncond[k]) -
-                     log((double)nrowt[k][i]) - log((double)ncolt[k][j]) );
-
-        }/*THEN*/
-
-      }/*FOR*/
+      for (i = 0; i < *nr; i++) 
+        res += MI_PART(n[k][CMC(i, j, *nr)], nrowt[k][i], ncolt[k][j], ncond[k]);
 
   return res;
 
